---------------------------------------------------------------------------------
--
-- DATABASE AND SCHEMA DEFINITION UNCOMMENT IF NECESSARY
--
---------------------------------------------------------------------------------

-- Database: "FleXtat"

--DROP DATABASE "FleXtat";

--CREATE DATABASE "FleXtat"
--WITH OWNER = postgres
--       ENCODING = 'UNICODE'
--       TABLESPACE = pg_default;

-- Schema: "public"

-- DROP SCHEMA public;

--CREATE SCHEMA public
--  AUTHORIZATION postgres;
--GRANT ALL ON SCHEMA public TO postgres;
--COMMENT ON SCHEMA public IS 'Standard public schema';

---------------------------------------------------------------------------------
--
-- USER GROUPS DEFINITION
--
---------------------------------------------------------------------------------

-- Group: "admins"

--DROP GROUP admins;

CREATE Group admins
  WITH SYSID 100
  USER postgres;

-- Group: "editors"

--DROP GROUP editors;

CREATE Group editors
  WITH SYSID 101;

-- Group: "users"

--DROP GROUP users;

CREATE Group users
  WITH SYSID 102;

GRANT ALL ON SCHEMA public TO GROUP admins;
GRANT USAGE ON SCHEMA public TO GROUP editors;
GRANT USAGE ON SCHEMA public TO GROUP users;

---------------------------------------------------------------------------------
--
-- SEQUENCES DEFINITION
--
---------------------------------------------------------------------------------

-- Sequence: cis_cis_id

-- DROP SEQUENCE cis_cis_id;

CREATE SEQUENCE cis_cis_id
  INCREMENT 1
  MINVALUE 1
  MAXVALUE 9223372036854775807
  START 1
  CACHE 1;
ALTER TABLE cis_cis_id OWNER TO postgres;
GRANT SELECT, UPDATE ON TABLE cis_cis_id TO GROUP admins;

-- Sequence: cis_dat_id

-- DROP SEQUENCE cis_dat_id;

CREATE SEQUENCE cis_dat_id
  INCREMENT 1
  MINVALUE 1
  MAXVALUE 9223372036854775807
  START 1
  CACHE 1;
ALTER TABLE cis_dat_id OWNER TO postgres;
GRANT SELECT, UPDATE ON TABLE cis_dat_id TO GROUP admins;

-- Sequence: cis_fields_mapping_id

-- DROP SEQUENCE cis_fields_mapping_id;

CREATE SEQUENCE cis_fields_mapping_id
  INCREMENT 1
  MINVALUE 1
  MAXVALUE 9223372036854775807
  START 1
  CACHE 1;
ALTER TABLE cis_fields_mapping_id OWNER TO postgres;
GRANT SELECT, UPDATE ON TABLE cis_fields_mapping_id TO GROUP admins;
GRANT SELECT, UPDATE ON TABLE cis_fields_mapping_id TO GROUP editors;

-- Sequence: cis_report_id

-- DROP SEQUENCE cis_report_id;

CREATE SEQUENCE cis_report_id
  INCREMENT 1
  MINVALUE 1
  MAXVALUE 9223372036854775807
  START 1
  CACHE 1;
ALTER TABLE cis_report_id OWNER TO postgres;
GRANT SELECT, UPDATE ON TABLE cis_report_id TO GROUP admins;
GRANT SELECT, UPDATE ON TABLE cis_report_id TO GROUP editors;

-- Sequence: cis_tree_id

-- DROP SEQUENCE cis_tree_id;

CREATE SEQUENCE cis_tree_id
  INCREMENT 1
  MINVALUE 1
  MAXVALUE 9223372036854775807
  START 1
  CACHE 1;
ALTER TABLE cis_tree_id OWNER TO postgres;
GRANT SELECT, UPDATE ON TABLE cis_tree_id TO GROUP admins;
GRANT SELECT, UPDATE ON TABLE cis_tree_id TO GROUP editors;

-- Sequence: dat_user_id

-- DROP SEQUENCE dat_user_id;

CREATE SEQUENCE dat_user_id
  INCREMENT 1
  MINVALUE 1
  MAXVALUE 9223372036854775807
  START 1
  CACHE 1;
ALTER TABLE dat_user_id OWNER TO postgres;
GRANT SELECT, UPDATE ON TABLE dat_user_id TO GROUP admins;

---------------------------------------------------------------------------------
--
-- CUSTOM USER FUNCTIONS DEFINITION
--
---------------------------------------------------------------------------------

-- Function: cis_new_version(_idcis int4)

-- DROP FUNCTION cis_new_version(int4);

CREATE OR REPLACE FUNCTION cis_new_version(_idcis int4)
  RETURNS "timestamp" AS
$BODY$
DECLARE _time timestamp; 
BEGIN
    SELECT INTO _time validto FROM cis_dat WHERE idcis = _idcis AND (validfrom < current_timestamp) AND ((validto IS NULL) OR (validto > current_timestamp));
    IF (_time IS NULL)
    THEN
	_time := current_timestamp;    
    END IF;
    EXECUTE 'UPDATE cis_dat SET validto = ' || quote_literal(CAST(_time AS varchar)) || ' WHERE idcis = ' || CAST(_idcis AS varchar) || ' AND ' ||
	'(validfrom < ' || quote_literal(CAST(_time AS varchar)) || ') AND ((validto IS NULL) OR (validto > ' || quote_literal(CAST(_time AS varchar)) || '))';
    EXECUTE 'UPDATE cis_cis SET lastid = 0 WHERE id = ' || CAST(_idcis AS varchar);
    RETURN _time;
END;
$BODY$
  LANGUAGE 'plpgsql' VOLATILE;
ALTER FUNCTION cis_new_version(int4) OWNER TO postgres;
GRANT EXECUTE ON FUNCTION cis_new_version(int4) TO GROUP admins;

-- Function: clear_database()

-- DROP FUNCTION clear_database();

CREATE OR REPLACE FUNCTION clear_database()
  RETURNS "bit" AS
$BODY$
DECLARE _time timestamp;
	_idcis int;
BEGIN
    _time := current_timestamp;
    SELECT INTO _idcis id FROM cis_cis WHERE shortname = 'couty';
    EXECUTE 'UPDATE cis_dat SET validto = ' || quote_literal(CAST(_time AS varchar)) || 
	'WHERE (validfrom < ' || quote_literal(CAST(_time AS varchar)) || ') AND ((validto IS NULL) OR ' ||
	'(validto > ' || quote_literal(CAST(_time AS varchar)) || ')) AND (IDCis <> ' || CAST(_idcis AS varchar) || ')'; 
    DELETE FROM cis_messages;
    RETURN 1;
END;
$BODY$
  LANGUAGE 'plpgsql' VOLATILE;
ALTER FUNCTION clear_database() OWNER TO postgres;
GRANT EXECUTE ON FUNCTION clear_database() TO GROUP admins;

-- Function: get_next_id(_idcis int4)

-- DROP FUNCTION get_next_id(int4);

CREATE OR REPLACE FUNCTION get_next_id(_idcis int4)
  RETURNS int4 AS
$BODY$
DECLARE _temp int;
	_id int;
BEGIN
    SELECT INTO _temp lastid+1 FROM cis_cis WHERE id = _idcis;
    SELECT INTO _id iid+1 FROM cis_dat WHERE (idcis = _idcis)
	AND (validfrom < current_timestamp)
	AND ((validto IS NULL) OR (validto > current_timestamp))
	ORDER BY iid DESC
	LIMIT 1;
    IF (_temp >= _id) OR (_id IS NULL)
    THEN
	UPDATE cis_cis SET lastid = _temp WHERE id = _idcis;
	_id := _temp;
	RETURN _id;
    ELSE
	UPDATE cis_cis SET lastid = _id WHERE id = _idcis;
	RETURN _id;
    END IF;
END;
$BODY$
  LANGUAGE 'plpgsql' VOLATILE;
ALTER FUNCTION get_next_id(int4) OWNER TO postgres;
GRANT EXECUTE ON FUNCTION get_next_id(int4) TO GROUP admins;

---------------------------------------------------------------------------------
--
-- CUSTOM TRIGGER FUNCTIONS DEFINITION
--
---------------------------------------------------------------------------------

-- Function: cis_cis_delete()

-- DROP FUNCTION cis_cis_delete();

CREATE OR REPLACE FUNCTION cis_cis_delete()
  RETURNS "trigger" AS
$BODY$
DECLARE _viewname varchar(50);
	_insrule  varchar(50);
	_updrule  varchar(50);
	_delrule  varchar(50);
BEGIN
    _viewname := 'cis_' || OLD.shortname;
    _insrule  := _viewname || '_insert_rule';
    _updrule  := _viewname || '_update_rule';
    _delrule  := _viewname || '_delete_rule';
    --
    --DELETE ALL TABLE ROWS, VIEWS AND RULES ASSIGNED TO CIS_...
    --
    DELETE FROM cis_dat WHERE idcis = (SELECT id FROM cis_cis WHERE shortname = OLD.shortname);
    EXECUTE 'DROP RULE ' || quote_ident(_insrule) || ' ON ' || quote_ident(_viewname);
    EXECUTE 'DROP RULE ' || quote_ident(_updrule) || ' ON ' || quote_ident(_viewname);
    EXECUTE 'DROP RULE ' || quote_ident(_delrule) || ' ON ' || quote_ident(_viewname);
    EXECUTE 'DROP VIEW ' || quote_ident(_viewname);	
    RETURN OLD;
END;
$BODY$
  LANGUAGE 'plpgsql' VOLATILE;
ALTER FUNCTION cis_cis_delete() OWNER TO postgres;
GRANT EXECUTE ON FUNCTION cis_cis_delete() TO GROUP admins;

-- Function: cis_cis_insert()

-- DROP FUNCTION cis_cis_insert();

CREATE OR REPLACE FUNCTION cis_cis_insert()
  RETURNS "trigger" AS
$BODY$
DECLARE _updrule  varchar(50);
	_insrule  varchar(50);
	_delrule  varchar(50);
	_viewname varchar(50);
BEGIN
    _viewname := 'cis_' || NEW.shortname;
    _updrule  := 'cis_' || NEW.shortname || '_update_rule';
    _insrule  := 'cis_' || NEW.shortname || '_insert_rule';
    _delrule  := 'cis_' || NEW.shortname || '_delete_rule';
    --
    --CREATE VIEW FOR ACTIVE DATA IN CIS_... AND ASSIGN PRIVILEGS
    --
    EXECUTE 'CREATE VIEW ' || quote_ident(_viewname) || ' AS ' ||
	'SELECT iid as id, description FROM cis_dat ' ||
	'WHERE (idcis = ' || CAST(NEW.id AS varchar) || ') AND (validfrom < current_timestamp) AND ' ||
	'((validto IS NULL) OR (validto > current_timestamp)) ' ||
	'ORDER BY iid ASC';
    EXECUTE 'GRANT SELECT, RULE ON TABLE ' || quote_ident(_viewname) || '  TO GROUP users';
    EXECUTE 'GRANT SELECT, RULE ON TABLE ' || quote_ident(_viewname) || '  TO GROUP editors';
    EXECUTE 'GRANT SELECT, UPDATE, INSERT, DELETE, RULE ON TABLE ' || quote_ident(_viewname) || '  TO GROUP admins';
    --
    --CREATE RULES FOR CREATED VIEW FOR HANDLING INSERT, UPDATE AND DELETE
    --
    EXECUTE 'CREATE OR REPLACE RULE ' || quote_ident(_insrule) || ' AS ON INSERT TO ' || quote_ident(_viewname) || ' DO INSTEAD ' ||
	'INSERT INTO cis_dat (validfrom, idcis, iid, description) VALUES ( ' ||
	    '(SELECT DISTINCT validfrom FROM cis_dat WHERE (idcis = (SELECT id FROM cis_cis WHERE shortname = ' || quote_literal(NEW.shortname) || ')) AND (validfrom < current_timestamp) AND ((validto is NULL) OR (validto > current_timestamp))), ' ||
	    '(SELECT id FROM cis_cis WHERE shortname = ' || quote_literal(NEW.shortname) || '), ' ||
	    'NEW.id, ' ||
	    'NEW.description)';
    EXECUTE 'CREATE OR REPLACE RULE ' || quote_ident(_updrule) || ' AS ON UPDATE TO ' || quote_ident(_viewname) || ' DO INSTEAD ' ||
	'UPDATE cis_dat SET description = NEW.description WHERE ' ||
	    '(iid = NEW.id) AND (idcis = (SELECT id FROM cis_cis WHERE shortname = ' || quote_literal(NEW.shortname) ||
	    ')) AND (validfrom < current_timestamp) AND ((validto is NULL) OR (validto > current_timestamp))';
    EXECUTE 'CREATE OR REPLACE RULE ' || quote_ident(_delrule) || ' AS ON DELETE TO ' || quote_ident(_viewname) || ' DO INSTEAD ' ||
	'DELETE FROM cis_dat WHERE ' ||
	    '(iid = OLD.id) AND (idcis = (SELECT id FROM cis_cis WHERE shortname = ' || quote_literal(NEW.shortname) ||
	    '))';
    RETURN NULL;
END;
$BODY$
  LANGUAGE 'plpgsql' VOLATILE;
ALTER FUNCTION cis_cis_insert() OWNER TO postgres;
GRANT EXECUTE ON FUNCTION cis_cis_insert() TO GROUP admins;

-- Function: cis_cis_update()

-- DROP FUNCTION cis_cis_update();

CREATE OR REPLACE FUNCTION cis_cis_update()
  RETURNS "trigger" AS
$BODY$
DECLARE _oldname varchar(50);
	_newname varchar(50);
	_oldinsr varchar(50);
	_newinsr varchar(50);
	_oldupdr varchar(50);
	_newupdr varchar(50);
	_olddelr varchar(50);
	_newdelr varchar(50);
BEGIN
IF (NEW.shortname <> OLD.shortname)
THEN
    _oldname := 'cis_' || OLD.shortname;
    _newname := 'cis_' || NEW.shortname;
    _oldinsr := _oldname || '_insert_rule';
    _newinsr := _newname || '_insert_rule';
    _oldupdr := _oldname || '_update_rule';
    _newupdr := _newname || '_update_rule';
    _olddelr := _oldname || '_delete_rule';
    _newdelr := _newname || '_delete_rule';
    --
    --RULE CANNOT BE RENAMED, SO DROP THEM, RENAME VIEW AND RECREATE RULES FOR VIEW VITH CORRESPONDING NAME
    --
    EXECUTE 'DROP RULE ' || quote_ident(_oldinsr) || ' ON ' || quote_ident(_oldname);
    EXECUTE 'DROP RULE ' || quote_ident(_oldupdr) || ' ON ' || quote_ident(_oldname);
    EXECUTE 'DROP RULE ' || quote_ident(_olddelr) || ' ON ' || quote_ident(_oldname);
    EXECUTE 'ALTER TABLE ' || quote_ident(_oldname) || ' RENAME TO ' || quote_ident(_newname);
    EXECUTE 'CREATE OR REPLACE RULE ' || quote_ident(_newinsr) || ' AS ON INSERT TO ' || quote_ident(_newname) || ' DO INSTEAD ' ||
	'INSERT INTO cis_dat (validfrom, idcis, iid, description) VALUES ( ' ||
	    '(SELECT DISTINCT validfrom FROM cis_dat WHERE (idcis = (SELECT id FROM cis_cis WHERE shortname = ' || quote_literal(NEW.shortname) || ')) AND (validfrom < current_timestamp) AND ((validto is NULL) OR (validto > current_timestamp))), ' ||
	    '(SELECT id FROM cis_cis WHERE shortname = ' || quote_literal(NEW.shortname) || '), ' ||
	    'NEW.id, ' ||
	    'NEW.description)';
    EXECUTE 'CREATE OR REPLACE RULE ' || quote_ident(_newupdr) || ' AS ON UPDATE TO ' || quote_ident(_newname) || ' DO INSTEAD ' ||
	'UPDATE cis_dat SET description = NEW.description WHERE ' ||
	    '(iid = NEW.id) AND (idcis = (SELECT id FROM cis_cis WHERE shortname = ' || quote_literal(NEW.shortname) ||
	    ')) AND (validfrom < current_timestamp) AND ((validto is NULL) OR (validto > current_timestamp))';
    EXECUTE 'CREATE OR REPLACE RULE ' || quote_ident(_newdelr) || ' AS ON DELETE TO ' || quote_ident(_newname) || ' DO INSTEAD ' ||
	'DELETE FROM cis_dat WHERE ' ||
	    '(iid = OLD.id) AND (idcis = (SELECT id FROM cis_cis WHERE shortname = ' || quote_literal(NEW.shortname) ||
	    '))';
END IF;
RETURN NULL;
END;
$BODY$
  LANGUAGE 'plpgsql' VOLATILE;
ALTER FUNCTION cis_cis_update() OWNER TO postgres;
GRANT EXECUTE ON FUNCTION cis_cis_update() TO GROUP admins;

-- Function: cis_dat_delete()

-- DROP FUNCTION cis_dat_delete();

CREATE OR REPLACE FUNCTION cis_dat_delete()
  RETURNS "trigger" AS
$BODY$
DECLARE _time timestamp;
BEGIN
    _time := cis_new_version(OLD.idcis);
    EXECUTE 'INSERT INTO cis_dat (idcis, iid, description) ' ||
	'SELECT idcis, iid, description FROM cis_dat ' ||
	'WHERE (idcis = ' || CAST(OLD.idcis AS varchar) || ') AND (iid <> ' || OLD.iid || ') ' ||
	' AND (validto = ' || quote_literal(CAST(_time AS varchar)) || ') ORDER BY iid ASC';
    RETURN NULL;
END;
$BODY$
  LANGUAGE 'plpgsql' VOLATILE STRICT;
ALTER FUNCTION cis_dat_delete() OWNER TO postgres;
GRANT EXECUTE ON FUNCTION cis_dat_delete() TO GROUP admins;

-- Function: cis_dat_insert()

-- DROP FUNCTION cis_dat_insert();

CREATE OR REPLACE FUNCTION cis_dat_insert()
  RETURNS "trigger" AS
$BODY$
DECLARE _time timestamp;
	_iiid int;
BEGIN
_time := NULL;
--IF (EXISTS (SELECT * FROM cis_dat WHERE (description = NEW.description) AND (idcis = NEW.idcis) AND (validfrom < current_timestamp) AND ((validto IS NULL) OR (validto > current_timestamp))))
--THEN
    --
    --DESCRIPTION EXISTS, SO DO NOT NEED TO DO ANYTHING
    --
--    RETURN NULL;
--ELSE
    --
    --INSERT NEW ONE
    -- If validFrom IS NOT SUPPLIED, THAN ASSIGN CURRENT
    --
    IF (_time IS NULL)
    THEN
	SELECT DISTINCT INTO _time validfrom FROM cis_dat WHERE (idcis = NEW.idcis) AND (validto IS NULL OR validto > current_timestamp) AND (validfrom < current_timestamp)
	    ORDER BY validfrom DESC LIMIT 1;
	IF (_time IS NULL)
	THEN
	    SELECT DISTINCT INTO _time validto FROM cis_dat WHERE (idcis = NEW.idcis) ORDER BY validto DESC LIMIT 1;
	    IF (_time IS NULL)
	    THEN
		_time := current_timestamp;
	    END IF;
	END IF;
    END IF;
    IF (NEW.iid > 0)
    THEN
	SELECT INTO _iiid lastid FROM cis_cis WHERE id = NEW.idcis;
	IF (_iiid >= NEW.iid)
	THEN
	    _iiid := get_next_id(NEW.idcis);
	    NEW.iid := _iiid;
	ELSE
            EXECUTE 'UPDATE cis_cis SET lastid = ' || CAST(NEW.iid AS varchar) || ' WHERE id = ' || CAST(NEW.idcis AS varchar);
	END IF;
    ELSE
        _iiid := get_next_id(NEW.idcis);
	NEW.iid := _iiid;
    END IF;
    NEW.validfrom := _time;
    RETURN NEW;
--END IF;
END;
$BODY$
  LANGUAGE 'plpgsql' VOLATILE;
ALTER FUNCTION cis_dat_insert() OWNER TO postgres;
GRANT EXECUTE ON FUNCTION cis_dat_insert() TO GROUP admins;

-- Function: cis_dat_update()

-- DROP FUNCTION cis_dat_update();

CREATE OR REPLACE FUNCTION cis_dat_update()
  RETURNS "trigger" AS
$BODY$
DECLARE _time timestamp;
BEGIN
    --
    --NEED TO CREATE NEW VERSION WITH UPDATED COLUMNS
    --
    IF NEW.description <> OLD.description
    THEN
	_time := cis_new_version(OLD.idcis);
	EXECUTE 'INSERT INTO cis_dat (idcis, iid, description) ' ||
	    'SELECT idcis, iid, description FROM cis_dat ' ||
	    'WHERE (idcis = ' || CAST(OLD.idcis AS varchar) || ') AND (iid < ' || OLD.iid || ') ' ||
	    ' AND (validto = ' || quote_literal(CAST(_time AS varchar)) || ') ORDER BY iid ASC';
	EXECUTE 'INSERT INTO cis_dat (idcis, iid, description) VALUES (' || NEW.idcis || ', '
	    || NEW.iid || ', ' || quote_literal(NEW.description) || ')';
	EXECUTE 'INSERT INTO cis_dat (idcis, iid, description) ' ||
	    'SELECT idcis, iid, description FROM cis_dat ' ||
	    'WHERE (idcis = ' || CAST(OLD.idcis AS varchar) || ') AND (iid > ' || OLD.iid || ') ' ||
	    ' AND (validto = ' || quote_literal(CAST(_time AS varchar)) || ') ORDER BY iid ASC';
	RETURN NULL;
    ELSE
	RETURN NEW;
    END IF;
END;
$BODY$
  LANGUAGE 'plpgsql' VOLATILE;
ALTER FUNCTION cis_dat_update() OWNER TO postgres;
GRANT EXECUTE ON FUNCTION cis_dat_update() TO GROUP admins;

-- Function: cis_report_delete()

-- DROP FUNCTION cis_report_delete();

CREATE OR REPLACE FUNCTION cis_report_delete()
  RETURNS "trigger" AS
$BODY$
BEGIN
    DELETE FROM cis_tree where report_id = old.id;
    RETURN old;
END;
$BODY$
  LANGUAGE 'plpgsql' VOLATILE;
ALTER FUNCTION cis_report_delete() OWNER TO postgres;
GRANT EXECUTE ON FUNCTION cis_report_delete() TO GROUP admins;
GRANT EXECUTE ON FUNCTION cis_report_delete() TO GROUP editors;

-- Function: dat_user_delete()

-- DROP FUNCTION dat_user_delete();

CREATE OR REPLACE FUNCTION dat_user_delete()
  RETURNS "trigger" AS
$BODY$
BEGIN
    EXECUTE 'DROP USER ' || OLD.username;
    RETURN OLD;
END;
$BODY$
  LANGUAGE 'plpgsql' VOLATILE;
ALTER FUNCTION dat_user_delete() OWNER TO postgres;
GRANT EXECUTE ON FUNCTION dat_user_delete() TO GROUP admins;

-- Function: dat_user_insert()

-- DROP FUNCTION dat_user_insert();

CREATE OR REPLACE FUNCTION dat_user_insert()
  RETURNS "trigger" AS
$BODY$
BEGIN
    IF (NEW.usergroup = 'admins')
    THEN
	EXECUTE 'CREATE USER ' || NEW.username || ' WITH NOCREATEDB CREATEUSER ENCRYPTED PASSWORD ' ||
	    quote_literal(NEW.temppwd) || ' IN GROUP ' || NEW.usergroup;
    ELSE
	EXECUTE 'CREATE USER ' || NEW.username || ' WITH NOCREATEDB NOCREATEUSER ENCRYPTED PASSWORD ' ||
	    quote_literal(NEW.temppwd) || ' IN GROUP ' || NEW.usergroup;
    END IF;
    NEW.temppwd := NULL;
    RETURN NEW;
END;
$BODY$
  LANGUAGE 'plpgsql' VOLATILE;
ALTER FUNCTION dat_user_insert() OWNER TO postgres;
GRANT EXECUTE ON FUNCTION dat_user_insert() TO GROUP admins;

-- Function: dat_user_update()

-- DROP FUNCTION dat_user_update();

CREATE OR REPLACE FUNCTION dat_user_update()
  RETURNS "trigger" AS
$BODY$
BEGIN
    IF (NEW.username <> OLD.username)
    THEN
	EXECUTE 'ALTER USER ' || OLD.username || ' RENAME TO ' || NEW.username;
    END IF;
    IF (NEW.temppwd <> OLD.temppwd)
    THEN
	EXECUTE 'ALTER USER ' || NEW.username || ' WITH ENCRYPTED PASSWORD ' || quote_literal(NEW.temppwd);
    END IF;
    IF (NEW.usergroup <> OLD.usergroup)
    THEN
	EXECUTE 'ALTER GROUP ' || OLD.usergroup || ' DROP USER ' || OLD.username;
	EXECUTE 'ALTER GROUP ' || NEW.usergroup || ' ADD USER ' || NEW.username;
    END IF;
    NEW.temppwd := NULL;
    RETURN NEW;
END;
$BODY$
  LANGUAGE 'plpgsql' VOLATILE;
ALTER FUNCTION dat_user_update() OWNER TO postgres;
GRANT EXECUTE ON FUNCTION dat_user_update() TO GROUP admins;
GRANT EXECUTE ON FUNCTION dat_user_update() TO GROUP editors;
GRANT EXECUTE ON FUNCTION dat_user_update() TO GROUP users;

---------------------------------------------------------------------------------
--
-- TABLES DEFINITION
--
---------------------------------------------------------------------------------

-- Table: cis_cis

-- DROP TABLE cis_cis;

CREATE TABLE cis_cis
(
  id int4 NOT NULL DEFAULT nextval('public.cis_cis_id'::text),
  shortname varchar(25) NOT NULL,
  description varchar(200) NOT NULL,
  lastid int4 NOT NULL DEFAULT 0,
  CONSTRAINT cis_cis_pkey PRIMARY KEY (id)
) 
WITHOUT OIDS;
ALTER TABLE cis_cis OWNER TO postgres;
GRANT SELECT ON TABLE cis_cis TO GROUP users;
GRANT SELECT ON TABLE cis_cis TO GROUP editors;
GRANT SELECT, UPDATE, INSERT, DELETE, TRIGGER ON TABLE cis_cis TO GROUP admins;

-- Table: cis_county_cis_couty

-- DROP TABLE cis_county_cis_couty;

CREATE TABLE cis_county_cis_couty
(
  county_id int4 NOT NULL,
  couty_id int4 NOT NULL
) 
WITHOUT OIDS;
ALTER TABLE cis_county_cis_couty OWNER TO postgres;
GRANT SELECT, UPDATE, INSERT, DELETE ON TABLE cis_county_cis_couty TO GROUP admins;
GRANT SELECT ON TABLE cis_county_cis_couty TO GROUP editors;
GRANT SELECT ON TABLE cis_county_cis_couty TO GROUP users;

-- Table: cis_dat

-- DROP TABLE cis_dat;

CREATE TABLE cis_dat
(
  id int4 NOT NULL DEFAULT nextval('public.cis_dat_id'::text),
  validfrom timestamp DEFAULT ('now'::text)::timestamp(6) with time zone,
  validto timestamp,
  idcis int4 NOT NULL,
  iid int4,
  description varchar(200),
  CONSTRAINT cis_dat_pkey PRIMARY KEY (id),
  CONSTRAINT cis_cis_cis_dat FOREIGN KEY (idcis) REFERENCES cis_cis (id) ON UPDATE RESTRICT ON DELETE RESTRICT
) 
WITHOUT OIDS;
ALTER TABLE cis_dat OWNER TO postgres;
GRANT SELECT ON TABLE cis_dat TO GROUP users;
GRANT SELECT ON TABLE cis_dat TO GROUP editors;
GRANT SELECT, UPDATE, INSERT, DELETE, TRIGGER ON TABLE cis_dat TO GROUP admins;

-- Table: cis_fields_mapping

-- DROP TABLE cis_fields_mapping;

CREATE TABLE cis_fields_mapping
(
  id int4 NOT NULL DEFAULT nextval('public.cis_fields_mapping_id'::text),
  caption varchar(50),
  name varchar(50),
  "type" varchar(50),
  CONSTRAINT cis_fields_mapping_pkey PRIMARY KEY (id)
) 
WITHOUT OIDS;
ALTER TABLE cis_fields_mapping OWNER TO postgres;
GRANT SELECT ON TABLE cis_fields_mapping TO GROUP users;
GRANT SELECT ON TABLE cis_fields_mapping TO GROUP editors;
GRANT SELECT, UPDATE, INSERT, DELETE ON TABLE cis_fields_mapping TO GROUP admins;

-- Table: cis_messages

-- DROP TABLE cis_messages;

CREATE TABLE cis_messages
(
  id varchar(200) NOT NULL,
  description varchar(200),
  CONSTRAINT cis_messages_pkey PRIMARY KEY (id)
) 
WITHOUT OIDS;
ALTER TABLE cis_messages OWNER TO postgres;
GRANT SELECT ON TABLE cis_messages TO GROUP users;
GRANT SELECT ON TABLE cis_messages TO GROUP editors;
GRANT SELECT, UPDATE, INSERT, DELETE ON TABLE cis_messages TO GROUP admins;

-- Table: cis_report

-- DROP TABLE cis_report;

CREATE TABLE cis_report
(
  id int4 NOT NULL DEFAULT nextval('public.cis_report_id'::text),
  detail varchar(500),
  report_name varchar(40) NOT NULL,
  report_file bytea,
  scriptlet_name varchar(40),
  scriptlet_type int4,
  scriptlet_file bytea,
  ds_type varchar(20),
  CONSTRAINT cis_report_pkey PRIMARY KEY (id)
) 
WITHOUT OIDS;
ALTER TABLE cis_report OWNER TO postgres;
GRANT SELECT ON TABLE cis_report TO GROUP users;
GRANT SELECT, UPDATE, INSERT, DELETE, TRIGGER ON TABLE cis_report TO GROUP editors;
GRANT SELECT, UPDATE, INSERT, DELETE, TRIGGER ON TABLE cis_report TO GROUP admins;

-- Constraint: cis_report_report_name_unique

-- ALTER TABLE cis_report DROP CONSTRAINT cis_report_report_name_unique

ALTER TABLE cis_report
  ADD CONSTRAINT cis_report_report_name_unique UNIQUE(report_name);

-- Table: cis_tree

-- DROP TABLE cis_tree;

CREATE TABLE cis_tree
(
  id int4 NOT NULL DEFAULT nextval('public.cis_tree_id'::text),
  popis varchar(40) NOT NULL,
  upper_id int4,
  report_id int4,
  CONSTRAINT cis_tree_pkey PRIMARY KEY (id),
  CONSTRAINT cis_tree_cis_report_fkey FOREIGN KEY (report_id) REFERENCES cis_report (id) ON UPDATE NO ACTION ON DELETE NO ACTION,
  CONSTRAINT cis_tree_cis_tree_fkey FOREIGN KEY (upper_id) REFERENCES cis_tree (id) ON UPDATE NO ACTION ON DELETE NO ACTION
) 
WITHOUT OIDS;
ALTER TABLE cis_tree OWNER TO postgres;
GRANT SELECT ON TABLE cis_tree TO GROUP users;
GRANT SELECT, UPDATE, INSERT, DELETE ON TABLE cis_tree TO GROUP editors;
GRANT SELECT, UPDATE, INSERT, DELETE ON TABLE cis_tree TO GROUP admins;

-- Table: dat_user

-- DROP TABLE dat_user;

CREATE TABLE dat_user
(
  id int4 NOT NULL DEFAULT nextval('public.dat_user_id'::text),
  username varchar(20) NOT NULL,
  fullname varchar(25),
  fullsurname varchar(30),
  title varchar(20),
  address varchar(30),
  phone1 varchar(25),
  phone2 varchar(25),
  phone3 varchar(25),
  email varchar(64),
  city varchar(30),
  zip varchar(6),
  temppwd varchar(30),
  usergroup varchar(10),
  CONSTRAINT dat_user_pkey PRIMARY KEY (id)
) 
WITHOUT OIDS;
ALTER TABLE dat_user OWNER TO postgres;
GRANT SELECT, UPDATE, TRIGGER ON TABLE dat_user TO GROUP users;
GRANT SELECT, UPDATE, TRIGGER ON TABLE dat_user TO GROUP editors;
GRANT SELECT, UPDATE, INSERT, DELETE, TRIGGER ON TABLE dat_user TO GROUP admins;

-- Constraint: dat_user_username_unique

-- ALTER TABLE dat_user DROP CONSTRAINT dat_user_username_unique

ALTER TABLE dat_user
  ADD CONSTRAINT dat_user_username_unique UNIQUE(username);

-- Table: dat_stat

-- DROP TABLE dat_stat;

CREATE TABLE dat_stat (
	application_id int NOT NULL,
	degree_id int NOT NULL ,
	faculty_id int NOT NULL ,
	submit_stamp timestamp NOT NULL ,
	create_stamp timestamp NOT NULL ,
	admitted bool NOT NULL ,
	admitted_course_id int ,
	state int NOT NULL ,
	operator_added bool NOT NULL ,
	birth_date timestamp NOT NULL ,
	gender_id int ,
	marital_status_id int ,
	disability_id int ,
	citizenship_id int ,
	nationality_id int NOT NULL ,
	county_id int ,
	couty_id int4 ,
	zip char (5) ,
	edu_type_id int ,
	country_id int NOT NULL ,
	edu_completion_year char (4) ,
	edu_title varchar (16)  NULL ,
	prev_occupation_id int ,
	school_id int NOT NULL ,
	gpa1 numeric(3, 2) NULL ,
	gpa2 numeric(3, 2) NULL ,
	gpa3 numeric(3, 2) NULL ,
	gpa4 numeric(3, 2) NULL ,
	gpa5 numeric(3, 2) NULL ,
	gpa_final numeric(3, 2) NULL ,
	prefered_course_id int NOT NULL ,
	prefered_course_preference int NULL ,
	exam_subject_id int NOT NULL ,
	exam_result int NULL ,
	school_type_id int NOT NULL ,
	school_zip char (5)  NOT NULL ,
	ranked_contest_id int ,
	approved_contest_rank_id int ,
	approved_contest_rank_bonus int ,
	approved_contest_rank_accepted bool NULL
) 
WITHOUT OIDS;
ALTER TABLE dat_stat OWNER TO postgres;
GRANT SELECT ON TABLE dat_stat TO GROUP users;
GRANT SELECT ON TABLE dat_stat TO GROUP editors;
GRANT SELECT, INSERT ON TABLE dat_stat TO GROUP admins;

---------------------------------------------------------------------------------
--
-- VIEWS DEFINITION
--
---------------------------------------------------------------------------------

-- View: "dat_users"

-- DROP VIEW dat_users;

CREATE OR REPLACE VIEW dat_users AS 
 SELECT dat_user.id, dat_user.username, dat_user.fullname, dat_user.fullsurname, dat_user.title, dat_user.address, dat_user.phone1, dat_user.phone2, dat_user.phone3, dat_user.email, dat_user.city, dat_user.zip, dat_user.temppwd AS userpwd, dat_user.usergroup
   FROM dat_user;

ALTER TABLE dat_users OWNER TO postgres;
GRANT SELECT, UPDATE, INSERT, DELETE, RULE ON TABLE dat_users TO GROUP admins;
GRANT SELECT, UPDATE, RULE ON TABLE dat_users TO GROUP editors;
GRANT SELECT, UPDATE, RULE ON TABLE dat_users TO GROUP users;

---------------------------------------------------------------------------------
--
-- RULES DEFINITION
--
---------------------------------------------------------------------------------

-- Rule: "dat_users_delete_rule ON dat_users"

-- DROP RULE dat_users_delete_rule ON dat_users;

CREATE OR REPLACE RULE dat_users_delete_rule AS
    ON DELETE TO dat_users DO INSTEAD  DELETE FROM dat_user
  WHERE dat_user.id = old.id;

-- Rule: "dat_users_insert_rule ON dat_users"

-- DROP RULE dat_users_insert_rule ON dat_users;

CREATE OR REPLACE RULE dat_users_insert_rule AS
    ON INSERT TO dat_users DO INSTEAD  INSERT INTO dat_user (username, fullname, fullsurname, title, address, phone1, phone2, phone3, email, city, zip, temppwd, usergroup) 
  VALUES (new.username, new.fullname, new.fullsurname, new.title, new.address, new.phone1, new.phone2, new.phone3, new.email, new.city, new.zip, new.userpwd, new.usergroup);

-- Rule: "dat_users_update_rule ON dat_users"

-- DROP RULE dat_users_update_rule ON dat_users;

CREATE OR REPLACE RULE dat_users_update_rule AS
    ON UPDATE TO dat_users DO INSTEAD  UPDATE dat_user SET username = new.username, fullname = new.fullname, fullsurname = new.fullsurname, title = new.title, address = new.address, phone1 = new.phone1, phone2 = new.phone2, phone3 = new.phone3, email = new.email, city = new.city, zip = new.zip, temppwd = new.userpwd, usergroup = new.usergroup
  WHERE dat_user.id = old.id;

---------------------------------------------------------------------------------
--
-- TRIGGERS DEFINITION
--
---------------------------------------------------------------------------------

-- Trigger: cis_cis_delete_trigger on cis_cis

-- DROP TRIGGER cis_cis_delete_trigger ON cis_cis;

CREATE TRIGGER cis_cis_delete_trigger
  AFTER DELETE
  ON cis_cis
  FOR EACH ROW
  EXECUTE PROCEDURE cis_cis_delete();

-- Trigger: cis_cis_insert_trigger on cis_cis

-- DROP TRIGGER cis_cis_insert_trigger ON cis_cis;

CREATE TRIGGER cis_cis_insert_trigger
  AFTER INSERT
  ON cis_cis
  FOR EACH ROW
  EXECUTE PROCEDURE cis_cis_insert();

-- Trigger: cis_cis_update_trigger on cis_cis

-- DROP TRIGGER cis_cis_update_trigger ON cis_cis;

CREATE TRIGGER cis_cis_update_trigger
  AFTER UPDATE
  ON cis_cis
  FOR EACH ROW
  EXECUTE PROCEDURE cis_cis_update();

-- Trigger: cis_dat_delete_trigger on cis_dat

-- DROP TRIGGER cis_dat_delete_trigger ON cis_dat;

CREATE TRIGGER cis_dat_delete_trigger
  BEFORE DELETE
  ON cis_dat
  FOR EACH ROW
  EXECUTE PROCEDURE cis_dat_delete();

-- Trigger: cis_dat_insert_trigger on cis_dat

-- DROP TRIGGER cis_dat_insert_trigger ON cis_dat;

CREATE TRIGGER cis_dat_insert_trigger
  BEFORE INSERT
  ON cis_dat
  FOR EACH ROW
  EXECUTE PROCEDURE cis_dat_insert();

-- Trigger: cis_dat_update_trigger on cis_dat

-- DROP TRIGGER cis_dat_update_trigger ON cis_dat;

CREATE TRIGGER cis_dat_update_trigger
  BEFORE UPDATE
  ON cis_dat
  FOR EACH ROW
  EXECUTE PROCEDURE cis_dat_update();

-- Trigger: cis_report_delete_trigger on cis_report

-- DROP TRIGGER cis_report_delete_trigger ON cis_report;

CREATE TRIGGER cis_report_delete_trigger
  BEFORE DELETE
  ON cis_report
  FOR EACH ROW
  EXECUTE PROCEDURE cis_report_delete();

-- Trigger: dat_user_delete_trigger on dat_user

-- DROP TRIGGER dat_user_delete_trigger ON dat_user;

CREATE TRIGGER dat_user_delete_trigger
  BEFORE DELETE
  ON dat_user
  FOR EACH ROW
  EXECUTE PROCEDURE dat_user_delete();

-- Trigger: dat_user_insert_trigger on dat_user

-- DROP TRIGGER dat_user_insert_trigger ON dat_user;

CREATE TRIGGER dat_user_insert_trigger
  BEFORE INSERT
  ON dat_user
  FOR EACH ROW
  EXECUTE PROCEDURE dat_user_insert();

-- Trigger: dat_user_update_trigger on dat_user

-- DROP TRIGGER dat_user_update_trigger ON dat_user;

CREATE TRIGGER dat_user_update_trigger
  BEFORE UPDATE
  ON dat_user
  FOR EACH ROW
  EXECUTE PROCEDURE dat_user_update();

---------------------------------------------------------------------------------
--
-- PLACE FOR INSERTING VALUES (LIKE CIS, USERS etc...)
--
---------------------------------------------------------------------------------

INSERT INTO cis_cis (shortname, description) VALUES ('admitted_course','selnk predmetov');
INSERT INTO cis_cis (shortname, description) VALUES ('application_states','selnk stavov prihlok');
INSERT INTO cis_cis (shortname, description) VALUES ('approved_contest_rank','selnk hodnotenia sa');
INSERT INTO cis_cis (shortname, description) VALUES ('citizenship','selnk obianstva');
INSERT INTO cis_cis (shortname, description) VALUES ('country','selnk ttov');
INSERT INTO cis_cis (shortname, description) VALUES ('county','selnk okresov');
INSERT INTO cis_cis (shortname, description) VALUES ('couty','selnk krajov');
INSERT INTO cis_cis (shortname, description) VALUES ('degree','selnk stupa tdia');
INSERT INTO cis_cis (shortname, description) VALUES ('disability','selnk zdravotnej spsobilosti');
INSERT INTO cis_cis (shortname, description) VALUES ('edu_type','selnk typu tdia');
INSERT INTO cis_cis (shortname, description) VALUES ('exam_subject','selnk predmetov skok');
INSERT INTO cis_cis (shortname, description) VALUES ('faculty','selnk faklt');
INSERT INTO cis_cis (shortname, description) VALUES ('gender','selnk pohlav');
INSERT INTO cis_cis (shortname, description) VALUES ('marital_status','selnk rodinnho stavu');
INSERT INTO cis_cis (shortname, description) VALUES ('nationality','selnk nrodnost');
INSERT INTO cis_cis (shortname, description) VALUES ('prefered_course','selnk preferovanho odboru');
INSERT INTO cis_cis (shortname, description) VALUES ('prev_occupation','selnk predchdzajceho psobiska');
INSERT INTO cis_cis (shortname, description) VALUES ('ranked_contest','selnk sa');
INSERT INTO cis_cis (shortname, description) VALUES ('school','selnk kl');
INSERT INTO cis_cis (shortname, description) VALUES ('school_type','selnk typov kl');

INSERT INTO cis_couty (id, description) VALUES (1,'Bratislavsk');
INSERT INTO cis_couty (id, description) VALUES (2,'Treniansky');
INSERT INTO cis_couty (id, description) VALUES (3,'Trnavsk');
INSERT INTO cis_couty (id, description) VALUES (4,'Koick');
INSERT INTO cis_couty (id, description) VALUES (5,'Preovsk');
INSERT INTO cis_couty (id, description) VALUES (6,'Nitriansky');
INSERT INTO cis_couty (id, description) VALUES (7,'ilinsk');
INSERT INTO cis_couty (id, description) VALUES (8,'Banskobystrick');
INSERT INTO cis_couty (id, description) VALUES (9,'');

INSERT INTO cis_county_cis_couty (county_id, couty_id) VALUES (1,1);
INSERT INTO cis_county_cis_couty (county_id, couty_id) VALUES (2,1);
INSERT INTO cis_county_cis_couty (county_id, couty_id) VALUES (3,1);
INSERT INTO cis_county_cis_couty (county_id, couty_id) VALUES (4,1);
INSERT INTO cis_county_cis_couty (county_id, couty_id) VALUES (5,1);
INSERT INTO cis_county_cis_couty (county_id, couty_id) VALUES (6,1);
INSERT INTO cis_county_cis_couty (county_id, couty_id) VALUES (7,1);
INSERT INTO cis_county_cis_couty (county_id, couty_id) VALUES (8,1);
INSERT INTO cis_county_cis_couty (county_id, couty_id) VALUES (9,3);
INSERT INTO cis_county_cis_couty (county_id, couty_id) VALUES (10,3);
INSERT INTO cis_county_cis_couty (county_id, couty_id) VALUES (11,3);
INSERT INTO cis_county_cis_couty (county_id, couty_id) VALUES (12,3);
INSERT INTO cis_county_cis_couty (county_id, couty_id) VALUES (13,3);
INSERT INTO cis_county_cis_couty (county_id, couty_id) VALUES (14,3);
INSERT INTO cis_county_cis_couty (county_id, couty_id) VALUES (15,3);
INSERT INTO cis_county_cis_couty (county_id, couty_id) VALUES (16,2);
INSERT INTO cis_county_cis_couty (county_id, couty_id) VALUES (17,2);
INSERT INTO cis_county_cis_couty (county_id, couty_id) VALUES (18,2);
INSERT INTO cis_county_cis_couty (county_id, couty_id) VALUES (19,2);
INSERT INTO cis_county_cis_couty (county_id, couty_id) VALUES (20,2);
INSERT INTO cis_county_cis_couty (county_id, couty_id) VALUES (21,2);
INSERT INTO cis_county_cis_couty (county_id, couty_id) VALUES (22,2);
INSERT INTO cis_county_cis_couty (county_id, couty_id) VALUES (23,2);
INSERT INTO cis_county_cis_couty (county_id, couty_id) VALUES (24,2);
INSERT INTO cis_county_cis_couty (county_id, couty_id) VALUES (25,6);
INSERT INTO cis_county_cis_couty (county_id, couty_id) VALUES (26,6);
INSERT INTO cis_county_cis_couty (county_id, couty_id) VALUES (27,6);
INSERT INTO cis_county_cis_couty (county_id, couty_id) VALUES (28,6);
INSERT INTO cis_county_cis_couty (county_id, couty_id) VALUES (29,6);
INSERT INTO cis_county_cis_couty (county_id, couty_id) VALUES (30,6);
INSERT INTO cis_county_cis_couty (county_id, couty_id) VALUES (31,6);
INSERT INTO cis_county_cis_couty (county_id, couty_id) VALUES (32,7);
INSERT INTO cis_county_cis_couty (county_id, couty_id) VALUES (33,7);
INSERT INTO cis_county_cis_couty (county_id, couty_id) VALUES (34,7);
INSERT INTO cis_county_cis_couty (county_id, couty_id) VALUES (35,7);
INSERT INTO cis_county_cis_couty (county_id, couty_id) VALUES (36,7);
INSERT INTO cis_county_cis_couty (county_id, couty_id) VALUES (37,7);
INSERT INTO cis_county_cis_couty (county_id, couty_id) VALUES (38,7);
INSERT INTO cis_county_cis_couty (county_id, couty_id) VALUES (39,7);
INSERT INTO cis_county_cis_couty (county_id, couty_id) VALUES (40,7);
INSERT INTO cis_county_cis_couty (county_id, couty_id) VALUES (41,7);
INSERT INTO cis_county_cis_couty (county_id, couty_id) VALUES (42,7);
INSERT INTO cis_county_cis_couty (county_id, couty_id) VALUES (43,8);
INSERT INTO cis_county_cis_couty (county_id, couty_id) VALUES (44,8);
INSERT INTO cis_county_cis_couty (county_id, couty_id) VALUES (45,8);
INSERT INTO cis_county_cis_couty (county_id, couty_id) VALUES (46,8);
INSERT INTO cis_county_cis_couty (county_id, couty_id) VALUES (47,8);
INSERT INTO cis_county_cis_couty (county_id, couty_id) VALUES (48,8);
INSERT INTO cis_county_cis_couty (county_id, couty_id) VALUES (49,8);
INSERT INTO cis_county_cis_couty (county_id, couty_id) VALUES (50,8);
INSERT INTO cis_county_cis_couty (county_id, couty_id) VALUES (51,8);
INSERT INTO cis_county_cis_couty (county_id, couty_id) VALUES (52,8);
INSERT INTO cis_county_cis_couty (county_id, couty_id) VALUES (53,8);
INSERT INTO cis_county_cis_couty (county_id, couty_id) VALUES (54,8);
INSERT INTO cis_county_cis_couty (county_id, couty_id) VALUES (55,8);
INSERT INTO cis_county_cis_couty (county_id, couty_id) VALUES (56,5);
INSERT INTO cis_county_cis_couty (county_id, couty_id) VALUES (57,5);
INSERT INTO cis_county_cis_couty (county_id, couty_id) VALUES (58,5);
INSERT INTO cis_county_cis_couty (county_id, couty_id) VALUES (59,5);
INSERT INTO cis_county_cis_couty (county_id, couty_id) VALUES (60,5);
INSERT INTO cis_county_cis_couty (county_id, couty_id) VALUES (61,5);
INSERT INTO cis_county_cis_couty (county_id, couty_id) VALUES (62,5);
INSERT INTO cis_county_cis_couty (county_id, couty_id) VALUES (63,5);
INSERT INTO cis_county_cis_couty (county_id, couty_id) VALUES (64,5);
INSERT INTO cis_county_cis_couty (county_id, couty_id) VALUES (65,5);
INSERT INTO cis_county_cis_couty (county_id, couty_id) VALUES (66,5);
INSERT INTO cis_county_cis_couty (county_id, couty_id) VALUES (67,5);
INSERT INTO cis_county_cis_couty (county_id, couty_id) VALUES (68,5);
INSERT INTO cis_county_cis_couty (county_id, couty_id) VALUES (69,4);
INSERT INTO cis_county_cis_couty (county_id, couty_id) VALUES (70,4);
INSERT INTO cis_county_cis_couty (county_id, couty_id) VALUES (71,4);
INSERT INTO cis_county_cis_couty (county_id, couty_id) VALUES (72,4);
INSERT INTO cis_county_cis_couty (county_id, couty_id) VALUES (73,4);
INSERT INTO cis_county_cis_couty (county_id, couty_id) VALUES (74,4);
INSERT INTO cis_county_cis_couty (county_id, couty_id) VALUES (75,4);
INSERT INTO cis_county_cis_couty (county_id, couty_id) VALUES (76,4);
INSERT INTO cis_county_cis_couty (county_id, couty_id) VALUES (77,4);
INSERT INTO cis_county_cis_couty (county_id, couty_id) VALUES (78,4);
INSERT INTO cis_county_cis_couty (county_id, couty_id) VALUES (79,4);
INSERT INTO cis_county_cis_couty (county_id, couty_id) VALUES (80,9);

INSERT INTO cis_fields_mapping VALUES (default,'Prihlka','application_id','java.lang.Integer');
INSERT INTO cis_fields_mapping VALUES (default,'Stupe tdia','degree_id','java.lang.String');      
INSERT INTO cis_fields_mapping VALUES (default,'Fakulta','faculty_id','java.lang.String');
INSERT INTO cis_fields_mapping VALUES (default,'Dtum podania','submit_stamp','java.lang.String');
INSERT INTO cis_fields_mapping VALUES (default,'Dtum vytvorenia','create_stamp','java.lang.String');
INSERT INTO cis_fields_mapping VALUES (default,'Stav prijatia','admitted','java.lang.String');
INSERT INTO cis_fields_mapping VALUES (default,'Prijat/ na odbor','admitted_course_id','java.lang.Boolean');
INSERT INTO cis_fields_mapping VALUES (default,'Dtum narodenia','birth_date','java.lang.Date');
INSERT INTO cis_fields_mapping VALUES (default,'Pohlavie','gender_id','java.lang.String');
INSERT INTO cis_fields_mapping VALUES (default,'Manelsk stav','marital_status_id','java.lang.String');
INSERT INTO cis_fields_mapping VALUES (default,'Zdravotn stav','disability_id','java.lang.String');
INSERT INTO cis_fields_mapping VALUES (default,'Nrodnos','nationality_id','java.lang.String');
INSERT INTO cis_fields_mapping VALUES (default,'Okres','county_id','java.lang.String');
INSERT INTO cis_fields_mapping VALUES (default,'Kraj','couty_id','java.lang.String');
INSERT INTO cis_fields_mapping VALUES (default,'PS','zip','java.lang.String');
INSERT INTO cis_fields_mapping VALUES (default,'Dosiahnut vzdelanie','edu_type_id','java.lang.String');
INSERT INTO cis_fields_mapping VALUES (default,'tt','country_id','java.lang.String');
INSERT INTO cis_fields_mapping VALUES (default,'Rok dokonenia tdia','edu_completion_year','java.lang.String');
INSERT INTO cis_fields_mapping VALUES (default,'Titul','edu_title','java.lang.String');
INSERT INTO cis_fields_mapping VALUES (default,'Predchdzajce zamestnanie','prev_occupation_id','java.lang.String');
INSERT INTO cis_fields_mapping VALUES (default,'Predchdzajca kola','school_id','java.lang.String');
INSERT INTO cis_fields_mapping VALUES (default,'Priemer za 1.rok tdia','gpa1','java.lang.Double');
INSERT INTO cis_fields_mapping VALUES (default,'Priemer za 2.rok tdia','gpa2','java.lang.Double');
INSERT INTO cis_fields_mapping VALUES (default,'Priemer za 3.rok tdia','gpa3','java.lang.Double');
INSERT INTO cis_fields_mapping VALUES (default,'Priemer za 4.rok tdia','gpa4','java.lang.Double');
INSERT INTO cis_fields_mapping VALUES (default,'Priemer za 5.rok tdia','gpa5','java.lang.Double');
INSERT INTO cis_fields_mapping VALUES (default,'Celkov priemer','gpa_final','java.lang.Double');
INSERT INTO cis_fields_mapping VALUES (default,'Preferovan odbor','prefered_course_id','java.lang.String');
INSERT INTO cis_fields_mapping VALUES (default,'Maturitn predmet','exam_subject_id','java.lang.String');
INSERT INTO cis_fields_mapping VALUES (default,'Vsledok z maturity','exam_result','java.lang.Integer');
INSERT INTO cis_fields_mapping VALUES (default,'Typ navtevovanej koly','school_type_id','java.lang.String');
INSERT INTO cis_fields_mapping VALUES (default,'PS koly','school_zip','java.lang.String');
INSERT INTO cis_fields_mapping VALUES (default,'Meno akceptovanej sae','ranked_contest_id','java.lang.String');
INSERT INTO cis_fields_mapping VALUES (default,'Umiestnenie v sai','approved_contest_rank_id','java.lang.String');
INSERT INTO cis_fields_mapping VALUES (default,'Poet pridelench bodov ku skke','approved_contest_rank_bonus','java.lang.Integer');

INSERT INTO dat_users (username, userpwd, usergroup) VALUES ('Admin','admin','admins');

INSERT INTO cis_tree VALUES (1,'tatistiky');

---------------------------------------------------------------------------------
--
-- FINAL TABLE VIEW
--
---------------------------------------------------------------------------------

-- View: "dat_stat_applications"

-- DROP VIEW dat_stat_applications;

CREATE OR REPLACE VIEW dat_stat_applications AS 
 SELECT dat_stat.application_id, ( SELECT cis_messages.description
           FROM cis_messages
          WHERE cis_messages.id::text = ((( SELECT cis_dat.description
                   FROM cis_dat
                  WHERE cis_dat.id = dat_stat.degree_id))::text)) AS degree_id, ( SELECT cis_messages.description
           FROM cis_messages
          WHERE cis_messages.id::text = ((( SELECT cis_dat.description
                   FROM cis_dat
                  WHERE cis_dat.id = dat_stat.faculty_id))::text)) AS faculty_id, dat_stat.submit_stamp, dat_stat.create_stamp, dat_stat.admitted, ( SELECT cis_messages.description
           FROM cis_messages
          WHERE cis_messages.id::text = ((( SELECT cis_dat.description
                   FROM cis_dat
                  WHERE cis_dat.id = dat_stat.admitted_course_id))::text)) AS admitted_course_id, dat_stat.state, dat_stat.operator_added, dat_stat.birth_date, ( SELECT cis_messages.description
           FROM cis_messages
          WHERE cis_messages.id::text = ((( SELECT cis_dat.description
                   FROM cis_dat
                  WHERE cis_dat.id = dat_stat.gender_id))::text)) AS gender_id, ( SELECT cis_messages.description
           FROM cis_messages
          WHERE cis_messages.id::text = ((( SELECT cis_dat.description
                   FROM cis_dat
                  WHERE cis_dat.id = dat_stat.marital_status_id))::text)) AS marital_status_id, ( SELECT cis_messages.description
           FROM cis_messages
          WHERE cis_messages.id::text = ((( SELECT cis_dat.description
                   FROM cis_dat
                  WHERE cis_dat.id = dat_stat.disability_id))::text)) AS disability_id, ( SELECT cis_dat.description
           FROM cis_dat
          WHERE cis_dat.id = dat_stat.citizenship_id) AS citizenship_id, ( SELECT cis_messages.description
           FROM cis_messages
          WHERE cis_messages.id::text = ((( SELECT cis_dat.description
                   FROM cis_dat
                  WHERE cis_dat.id = dat_stat.nationality_id))::text)) AS nationality_id, ( SELECT cis_dat.description
           FROM cis_dat
          WHERE cis_dat.id = dat_stat.county_id) AS county_id, ( SELECT cis_dat.description
           FROM cis_dat
          WHERE cis_dat.id = dat_stat.couty_id) AS couty_id, dat_stat.zip, ( SELECT cis_messages.description
           FROM cis_messages
          WHERE cis_messages.id::text = ((( SELECT cis_dat.description
                   FROM cis_dat
                  WHERE cis_dat.id = dat_stat.edu_type_id))::text)) AS edu_type_id, ( SELECT cis_messages.description
           FROM cis_messages
          WHERE cis_messages.id::text = ((( SELECT cis_dat.description
                   FROM cis_dat
                  WHERE cis_dat.id = dat_stat.country_id))::text)) AS country_id, dat_stat.edu_completion_year, dat_stat.edu_title, ( SELECT cis_messages.description
           FROM cis_messages
          WHERE cis_messages.id::text = ((( SELECT cis_dat.description
                   FROM cis_dat
                  WHERE cis_dat.id = dat_stat.prev_occupation_id))::text)) AS prev_occupation_id, ( SELECT cis_dat.description
           FROM cis_dat
          WHERE cis_dat.id = dat_stat.school_id) AS school_id, dat_stat.gpa1, dat_stat.gpa2, dat_stat.gpa3, dat_stat.gpa4, dat_stat.gpa5, dat_stat.gpa_final, ( SELECT cis_messages.description
           FROM cis_messages
          WHERE cis_messages.id::text = ((( SELECT cis_dat.description
                   FROM cis_dat
                  WHERE cis_dat.id = dat_stat.prefered_course_id))::text)) AS prefered_course_id, dat_stat.prefered_course_preference, ( SELECT cis_messages.description
           FROM cis_messages
          WHERE cis_messages.id::text = ((( SELECT cis_dat.description
                   FROM cis_dat
                  WHERE cis_dat.id = dat_stat.exam_subject_id))::text)) AS exam_subject_id, dat_stat.exam_result, ( SELECT cis_messages.description
           FROM cis_messages
          WHERE cis_messages.id::text = ((( SELECT cis_dat.description
                   FROM cis_dat
                  WHERE cis_dat.id = dat_stat.school_type_id))::text)) AS school_type_id, dat_stat.school_zip, ( SELECT cis_dat.description
           FROM cis_dat
          WHERE cis_dat.id = dat_stat.ranked_contest_id) AS ranked_contest_id, ( SELECT cis_dat.description
           FROM cis_dat
          WHERE cis_dat.id = dat_stat.approved_contest_rank_id) AS approved_contest_rank_id, dat_stat.approved_contest_rank_bonus, dat_stat.approved_contest_rank_accepted
   FROM dat_stat;

ALTER TABLE dat_stat_applications OWNER TO postgres;
GRANT SELECT, RULE ON TABLE dat_stat_applications TO GROUP users;
GRANT SELECT, RULE ON TABLE dat_stat_applications TO GROUP editors;
GRANT SELECT, RULE ON TABLE dat_stat_applications TO GROUP admins;

-- View: "dat_stat_insert"

-- DROP VIEW dat_stat_insert;

CREATE OR REPLACE VIEW dat_stat_insert AS 
 SELECT dat_stat.application_id, ( SELECT cis_dat.iid
           FROM cis_dat
          WHERE cis_dat.id = dat_stat.degree_id) AS degree_id, ( SELECT cis_dat.iid
           FROM cis_dat
          WHERE cis_dat.id = dat_stat.faculty_id) AS faculty_id, dat_stat.submit_stamp, dat_stat.create_stamp, dat_stat.admitted, ( SELECT cis_dat.iid
           FROM cis_dat
          WHERE cis_dat.id = dat_stat.admitted_course_id) AS admitted_course_id, dat_stat.state, dat_stat.operator_added, dat_stat.birth_date, ( SELECT cis_dat.iid
           FROM cis_dat
          WHERE cis_dat.id = dat_stat.gender_id) AS gender_id, ( SELECT cis_dat.iid
           FROM cis_dat
          WHERE cis_dat.id = dat_stat.marital_status_id) AS marital_status_id, ( SELECT cis_dat.iid
           FROM cis_dat
          WHERE cis_dat.id = dat_stat.disability_id) AS disability_id, ( SELECT cis_dat.iid
           FROM cis_dat
          WHERE cis_dat.id = dat_stat.citizenship_id) AS citizenship_id, ( SELECT cis_dat.iid
           FROM cis_dat
          WHERE cis_dat.id = dat_stat.nationality_id) AS nationality_id, ( SELECT cis_dat.iid
           FROM cis_dat
          WHERE cis_dat.id = dat_stat.county_id) AS county_id, ( SELECT cis_dat.iid
           FROM cis_dat
          WHERE cis_dat.id = dat_stat.couty_id) AS couty_id, dat_stat.zip, ( SELECT cis_dat.iid
           FROM cis_dat
          WHERE cis_dat.id = dat_stat.edu_type_id) AS edu_type_id, ( SELECT cis_dat.iid
           FROM cis_dat
          WHERE cis_dat.id = dat_stat.country_id) AS country_id, dat_stat.edu_completion_year, dat_stat.edu_title, ( SELECT cis_dat.iid
           FROM cis_dat
          WHERE cis_dat.id = dat_stat.prev_occupation_id) AS prev_occupation_id, ( SELECT cis_dat.iid
           FROM cis_dat
          WHERE cis_dat.id = dat_stat.school_id) AS school_id, dat_stat.gpa1, dat_stat.gpa2, dat_stat.gpa3, dat_stat.gpa4, dat_stat.gpa5, dat_stat.gpa_final, ( SELECT cis_dat.iid
           FROM cis_dat
          WHERE cis_dat.id = dat_stat.prefered_course_id) AS prefered_course_id, dat_stat.prefered_course_preference, ( SELECT cis_dat.iid
           FROM cis_dat
          WHERE cis_dat.id = dat_stat.exam_subject_id) AS exam_subject_id, dat_stat.exam_result, ( SELECT cis_dat.iid
           FROM cis_dat
          WHERE cis_dat.id = dat_stat.school_type_id) AS school_type_id, dat_stat.school_zip, ( SELECT cis_dat.iid
           FROM cis_dat
          WHERE cis_dat.id = dat_stat.ranked_contest_id) AS ranked_contest_id, ( SELECT cis_dat.iid
           FROM cis_dat
          WHERE cis_dat.id = dat_stat.approved_contest_rank_id) AS approved_contest_rank_id, dat_stat.approved_contest_rank_bonus, dat_stat.approved_contest_rank_accepted
   FROM dat_stat;

ALTER TABLE dat_stat_insert OWNER TO postgres;
GRANT SELECT, RULE ON TABLE dat_stat_insert TO GROUP users;
GRANT SELECT, RULE ON TABLE dat_stat_insert TO GROUP editors;
GRANT SELECT, DELETE, INSERT, UPDATE, RULE ON TABLE dat_stat_insert TO GROUP admins;

-- Rule: "dat_stat_insert_insert_rule ON dat_stat_insert"

-- DROP RULE dat_stat_insert_insert_rule ON dat_stat_insert;

CREATE OR REPLACE RULE dat_stat_insert_insert_rule AS
    ON INSERT TO dat_stat_insert DO INSTEAD  INSERT INTO dat_stat (application_id, degree_id, faculty_id, submit_stamp, create_stamp, admitted, admitted_course_id, state, operator_added, birth_date, gender_id, marital_status_id, disability_id, citizenship_id, nationality_id, county_id, couty_id, zip, edu_type_id, country_id, edu_completion_year, edu_title, prev_occupation_id, school_id, gpa1, gpa2, gpa3, gpa4, gpa5, gpa_final, prefered_course_id, prefered_course_preference, exam_subject_id, exam_result, school_type_id, school_zip, ranked_contest_id, approved_contest_rank_id, approved_contest_rank_bonus, approved_contest_rank_accepted) 
  VALUES (new.application_id, ( SELECT cis_dat.id
           FROM cis_dat
          WHERE cis_dat.iid = new.degree_id AND cis_dat.idcis = (( SELECT cis_cis.id
                   FROM cis_cis
                  WHERE cis_cis.shortname::text = 'degree'::text)) AND cis_dat.validfrom < 'now'::text::timestamp(6) with time zone AND (cis_dat.validto IS NULL OR cis_dat.validto > 'now'::text::timestamp(6) with time zone)), ( SELECT cis_dat.id
           FROM cis_dat
          WHERE cis_dat.iid = new.faculty_id AND cis_dat.idcis = (( SELECT cis_cis.id
                   FROM cis_cis
                  WHERE cis_cis.shortname::text = 'faculty'::text)) AND cis_dat.validfrom < 'now'::text::timestamp(6) with time zone AND (cis_dat.validto IS NULL OR cis_dat.validto > 'now'::text::timestamp(6) with time zone)), new.submit_stamp, new.create_stamp, new.admitted, ( SELECT cis_dat.id
           FROM cis_dat
          WHERE cis_dat.iid = new.admitted_course_id AND cis_dat.idcis = (( SELECT cis_cis.id
                   FROM cis_cis
                  WHERE cis_cis.shortname::text = 'admitted_course'::text)) AND cis_dat.validfrom < 'now'::text::timestamp(6) with time zone AND (cis_dat.validto IS NULL OR cis_dat.validto > 'now'::text::timestamp(6) with time zone)), new.state, new.operator_added, new.birth_date, ( SELECT cis_dat.id
           FROM cis_dat
          WHERE cis_dat.iid = new.gender_id AND cis_dat.idcis = (( SELECT cis_cis.id
                   FROM cis_cis
                  WHERE cis_cis.shortname::text = 'gender'::text)) AND cis_dat.validfrom < 'now'::text::timestamp(6) with time zone AND (cis_dat.validto IS NULL OR cis_dat.validto > 'now'::text::timestamp(6) with time zone)), ( SELECT cis_dat.id
           FROM cis_dat
          WHERE cis_dat.iid = new.marital_status_id AND cis_dat.idcis = (( SELECT cis_cis.id
                   FROM cis_cis
                  WHERE cis_cis.shortname::text = 'marital_status'::text)) AND cis_dat.validfrom < 'now'::text::timestamp(6) with time zone AND (cis_dat.validto IS NULL OR cis_dat.validto > 'now'::text::timestamp(6) with time zone)), ( SELECT cis_dat.id
           FROM cis_dat
          WHERE cis_dat.iid = new.disability_id AND cis_dat.idcis = (( SELECT cis_cis.id
                   FROM cis_cis
                  WHERE cis_cis.shortname::text = 'disability'::text)) AND cis_dat.validfrom < 'now'::text::timestamp(6) with time zone AND (cis_dat.validto IS NULL OR cis_dat.validto > 'now'::text::timestamp(6) with time zone)), ( SELECT cis_dat.id
           FROM cis_dat
          WHERE cis_dat.iid = new.citizenship_id AND cis_dat.idcis = (( SELECT cis_cis.id
                   FROM cis_cis
                  WHERE cis_cis.shortname::text = 'citizenship'::text)) AND cis_dat.validfrom < 'now'::text::timestamp(6) with time zone AND (cis_dat.validto IS NULL OR cis_dat.validto > 'now'::text::timestamp(6) with time zone)), ( SELECT cis_dat.id
           FROM cis_dat
          WHERE cis_dat.iid = new.nationality_id AND cis_dat.idcis = (( SELECT cis_cis.id
                   FROM cis_cis
                  WHERE cis_cis.shortname::text = 'nationality'::text)) AND cis_dat.validfrom < 'now'::text::timestamp(6) with time zone AND (cis_dat.validto IS NULL OR cis_dat.validto > 'now'::text::timestamp(6) with time zone)), ( SELECT cis_dat.id
           FROM cis_dat
          WHERE cis_dat.iid = new.county_id AND cis_dat.idcis = (( SELECT cis_cis.id
                   FROM cis_cis
                  WHERE cis_cis.shortname::text = 'county'::text)) AND cis_dat.validfrom < 'now'::text::timestamp(6) with time zone AND (cis_dat.validto IS NULL OR cis_dat.validto > 'now'::text::timestamp(6) with time zone)), ( SELECT cis_dat.id
           FROM cis_dat
          WHERE cis_dat.iid = (( SELECT cis_county_cis_couty.couty_id
                   FROM cis_county_cis_couty
                  WHERE cis_county_cis_couty.county_id = new.county_id)) AND cis_dat.idcis = (( SELECT cis_cis.id
                   FROM cis_cis
                  WHERE cis_cis.shortname::text = 'couty'::text)) AND cis_dat.validfrom < 'now'::text::timestamp(6) with time zone AND (cis_dat.validto IS NULL OR cis_dat.validto > 'now'::text::timestamp(6) with time zone)), new.zip, ( SELECT cis_dat.id
           FROM cis_dat
          WHERE cis_dat.iid = new.edu_type_id AND cis_dat.idcis = (( SELECT cis_cis.id
                   FROM cis_cis
                  WHERE cis_cis.shortname::text = 'edu_type'::text)) AND cis_dat.validfrom < 'now'::text::timestamp(6) with time zone AND (cis_dat.validto IS NULL OR cis_dat.validto > 'now'::text::timestamp(6) with time zone)), ( SELECT cis_dat.id
           FROM cis_dat
          WHERE cis_dat.iid = new.country_id AND cis_dat.idcis = (( SELECT cis_cis.id
                   FROM cis_cis
                  WHERE cis_cis.shortname::text = 'country'::text)) AND cis_dat.validfrom < 'now'::text::timestamp(6) with time zone AND (cis_dat.validto IS NULL OR cis_dat.validto > 'now'::text::timestamp(6) with time zone)), new.edu_completion_year, new.edu_title, ( SELECT cis_dat.id
           FROM cis_dat
          WHERE cis_dat.iid = new.prev_occupation_id AND cis_dat.idcis = (( SELECT cis_cis.id
                   FROM cis_cis
                  WHERE cis_cis.shortname::text = 'prev_occupation'::text)) AND cis_dat.validfrom < 'now'::text::timestamp(6) with time zone AND (cis_dat.validto IS NULL OR cis_dat.validto > 'now'::text::timestamp(6) with time zone)), ( SELECT cis_dat.id
           FROM cis_dat
          WHERE cis_dat.iid = new.school_id AND cis_dat.idcis = (( SELECT cis_cis.id
                   FROM cis_cis
                  WHERE cis_cis.shortname::text = 'school'::text)) AND cis_dat.validfrom < 'now'::text::timestamp(6) with time zone AND (cis_dat.validto IS NULL OR cis_dat.validto > 'now'::text::timestamp(6) with time zone)), new.gpa1, new.gpa2, new.gpa3, new.gpa4, new.gpa5, new.gpa_final, ( SELECT cis_dat.id
           FROM cis_dat
          WHERE cis_dat.iid = new.prefered_course_id AND cis_dat.idcis = (( SELECT cis_cis.id
                   FROM cis_cis
                  WHERE cis_cis.shortname::text = 'prefered_course'::text)) AND cis_dat.validfrom < 'now'::text::timestamp(6) with time zone AND (cis_dat.validto IS NULL OR cis_dat.validto > 'now'::text::timestamp(6) with time zone)), new.prefered_course_preference, ( SELECT cis_dat.id
           FROM cis_dat
          WHERE cis_dat.iid = new.exam_subject_id AND cis_dat.idcis = (( SELECT cis_cis.id
                   FROM cis_cis
                  WHERE cis_cis.shortname::text = 'exam_subject'::text)) AND cis_dat.validfrom < 'now'::text::timestamp(6) with time zone AND (cis_dat.validto IS NULL OR cis_dat.validto > 'now'::text::timestamp(6) with time zone)), new.exam_result, ( SELECT cis_dat.id
           FROM cis_dat
          WHERE cis_dat.iid = new.school_type_id AND cis_dat.idcis = (( SELECT cis_cis.id
                   FROM cis_cis
                  WHERE cis_cis.shortname::text = 'school_type'::text)) AND cis_dat.validfrom < 'now'::text::timestamp(6) with time zone AND (cis_dat.validto IS NULL OR cis_dat.validto > 'now'::text::timestamp(6) with time zone)), new.school_zip, ( SELECT cis_dat.id
           FROM cis_dat
          WHERE cis_dat.iid = new.ranked_contest_id AND cis_dat.idcis = (( SELECT cis_cis.id
                   FROM cis_cis
                  WHERE cis_cis.shortname::text = 'ranked_contest'::text)) AND cis_dat.validfrom < 'now'::text::timestamp(6) with time zone AND (cis_dat.validto IS NULL OR cis_dat.validto > 'now'::text::timestamp(6) with time zone)), ( SELECT cis_dat.id
           FROM cis_dat
          WHERE cis_dat.iid = new.approved_contest_rank_id AND cis_dat.idcis = (( SELECT cis_cis.id
                   FROM cis_cis
                  WHERE cis_cis.shortname::text = 'approved_contest_rank'::text)) AND cis_dat.validfrom < 'now'::text::timestamp(6) with time zone AND (cis_dat.validto IS NULL OR cis_dat.validto > 'now'::text::timestamp(6) with time zone)), new.approved_contest_rank_bonus, new.approved_contest_rank_accepted);
